-- This AI file may be distributed and used in AI's freely as long as credit is given.
-- Written by Dr. Azzy of iRO Loki
-- Version 1.00


require "./AI/USER_AI/SkillList.lua"


--[[
USAGE NOTES


GetHPPercent(id)
GetSPPercent(id)
Returns the current hp or sp of id as a percentage. This only works on things which the player can see the hp/sp of, obviously.


@@@@@@@@@@@@@@@@@@@@@@@@@
@@@ Monster Functions @@@
@@@@@@@@@@@@@@@@@@@@@@@@@

GetMobCount(id,range,[id2])
Returns the number of monsters within range cells of the actor id, and the number of said monsters which are currently targeting id. If id2 is specified, monsters which are targeting id2 will also be included in the second value returned. 
Useful for deciding whether to use anti-mob skills.
ex:
local mobcount,aggrocount = GetMobCount(MyID,1,GetV(V_OWNER,MyID))

GetBestBrandishTarget(id)
Returns the id of the best monster adjacent to the mercenary to use brandish spear on (that is, the target that will result in the most monsters being hit). 
Assumes level 1 brandish area. 

ex:
SkillObject(MyID,MySkillLevel,ML_BRANDISH,GetBestBrandishTarget(MyID))


@@@@@@@@@@@@@@@@@@@@@@@
@@@ Skill Functions @@@
@@@@@@@@@@@@@@@@@@@@@@@

GetAtkSkill(merctype)
Returns a list containing skill id, skill level, and sp consumption of the mercenary's single target offencive skill. Crash is not returned in this list, because the damage output spamming crash is lower than the damage output from using normal attacks. 

ex: 
local skill,level,sp = GetAtkSkill(GetV(V_MERTYPE,MyID))

GetMobSkill(merctype)
As GetAtkSkill(), but returns a fourth variable, target type:
0 - Use SkillObject() to cast the skill, targeted on mercenary (magnum break)
1 - Use SkillObject() to cast the skill, targeted on enemy
2 - Use SkillArea() to cast the skill (Arrow Shower)

ex
local skill,level,sp,targettype = GetMobSkill(GetV(V_MERTYPE,MyID))

GetQuickenSkill(merctype)
Returns a list containing the ID of the weapon quicken skill posessed by the mercenary, the skill level, the sp consumption, and the duration. 

ex
local skill,level,sp,duration = GetQuickenSkill(GetV(V_MERTYPE,MyID))

GetGuardSkill(merctype)
As GetQuickenSkill() only returns the ID of the guard or parry skill posessed by the mercenary. 
]]--


--########################
--### Friend Functions ###
--########################

function IsFriend(id)
	if (id==GetV(V_OWNER,MyID)) then
		return 1
	else
		if (MyFriends[id]~=nil) then
			return 1	
		end
	end
	return 0
end

function IsEnemy(id)  -- Placeholder for pvp functionality
	return 0
end

--########################
--### Monster functions###
--########################


function GetMobCount(id,range,id2)
	local actors = GetActors ()
	local mobcount=0
	local aggrocount=0
	local distance = 0
	local target = -1
	if (id2==nil) then
		id2 = 0
	end
	for i,v in ipairs(actors) do
		if (IsMonster(v) == 1) then
			distance = GetDistance2(id,v)
			if (distance <= range) then
				mobcount=mobcount+1
				target = GetV(V_TARGET,v)
				if (GetV(V_TARGET,v)==id or GetV(V_TARGET,v)==id2) then
					aggrocount=aggrocount+1
				end
			end
		end
	end
	return mobcount,aggrocount
end



function GetBestBrandishTarget(myid)
	local actors = GetActors()
	local enemies = {}
	local index = 1
	for i,v in ipairs(actors) do
		if (IsMonster(v)==1) then	
			enemies[index] = v
			index = index+1
		end
	end
	local mobs =0
	local maxmobs = 0
	local besttarget= -1
	for i,v in ipairs(enemies) do
		if (GetDistance2(myid,v) <=1) then --Is a target in range to brandish
			mobs = 0
			for a,b in ipairs (enemies) do
				if (GetDistance2(myid,b)<=1 and GetDistance2(v,b) <=1) then
					mobs=mobs+1
				elseif (GetDistance2(myid,b) ==2 and GetDistance3(myid,b) < 2.5 and GetDistance3(v,b) < 2.5)then -- sides, 2 cells to either side of target/merc
					mobs=mobs+1
				end
			end
			if (mobs > maxmobs) then
				maxmobs = mobs
				besttarget = v
			end
		end
	end
	return besttarget
end

function IsHiddenOnScreen(myid) --Currently disabled - assume there is always hidden on screen.
	return 1
end

function IsNotKS(myid,target)
	TraceAI("Checking for KS:"..target)
	local targettarget=GetV(V_TARGET,target)
	if (IsFriend(targettarget)==1 or targettarget==myid) then
		TraceAI("Not KS - target fighting friend")
		return 1
	elseif (targettarget > 0 and IsMonster(targettarget)~=1) then
		TraceAI("Is KS - target attacking other player"..targettarget)
		return 0
	else
		TraceAI("Not Targeted - seeing if anyone is targeting it")
		local actors = GetActors()
		for i,v in ipairs(actors) do
			if (IsMonster(v)~=1 and IsFriend(v)==0) then
				if (GetV(V_TARGET,target) == target) then
					TraceAI("Would be KS - dropping"..target)
					return 0
				end
			end
		end
	return 1
	end
end
--########################
--### HP/SP % functions###
--########################

function HPPercent(id)
	local maxHP=GetV(V_MAXHP,id)
	local curHP=GetV(V_HP,id)
	percHP=100*curHP/maxHP
	return percHP
end

function SPPercent(id)
	local maxSP=GetV(V_MAXSP,id)
	local curSP=GetV(V_SP,id)
	percSP=100*curSP/maxSP
	return percSP
end

-- returns a floating point distance between two actors, used in GetBestBrandishTarget()
function GetDistance3(id1,id2) 
	local x1, y1 = GetV (V_POSITION,id1)
	local x2, y2 = GetV (V_POSITION,id2)
	if (x1 == -1 or x2 == -1) then
		return -1
	end
	return math.sqrt((x1-x2)^2+(y1-y2)^2)
end

function GetOrbitNext(distance,step)
	local angle=math.rad(step*45)
	local destx=math.ceil(math.sin(angle)*distance)
	local desty=math.ceil(math.cos(angle)*distance)
	TraceAI("OrbitDestOffs: "..destx..","..desty) 
	return destx,desty
end

function BetterMoveToOwner(myid,range)
	local x,y = GetV(V_POSITION,myid)
	local ox,oy = GetV(V_POSITION,GetV(V_OWNER,myid))
	local destx,desty=0,0
	if (x > ox+range) then
		destx=ox+range
	elseif (x < ox - range) then
		destx=ox-range
	else
		destx=x
	end
	if (y > oy+range) then
		desty=oy+range
	elseif (y < oy - range) then
		desty=oy-range
	else
		desty=y
	end
	MyDestX,MyDestY=destx,desty
	Move(myid,MyDestX,MyDestY)
	return
end

function Closer(myid,ox,oy)
	x,y=GetV(V_POSITION,myid)
	newx,newy=0,0
	if (ox==x) then
		newx=x
	elseif (ox > x) then
		newx=ox-1
	else
		newx=ox+1
	end
	if (oy==y) then
		newy=y
	elseif (oy > y) then
		newy=oy-1
	else
		newy=oy+1
	end
	return newx,newy
end
function DiagonalDist(distance)
	return math.floor(math.sqrt(distance*distance))
end

--#########################
--### GetSkill functions###
--#########################

function GetAtkSkill(merctype)
	local skill = 0
	local level = 0
	local sp = 0
	if (merctype==1) then
		skill=MA_DOUBLE
		level=2
		sp=12
	elseif (merctype==5) then
		skill=MA_DOUBLE
		level=5
		sp=12
	elseif (merctype==6) then
		skill=MA_DOUBLE
		level=7
		sp=12
	elseif (merctype==9) then
		skill=MA_DOUBLE
		level=10
		sp=12
	elseif (merctype==11) then
		skill=ML_PIERCE
		level=1
		sp=7
	elseif (merctype==13) then
		skill=ML_PIERCE
		level=2
		sp=7
	elseif (merctype==15) then
		skill=ML_PIERCE
		level=5
		sp=7
	elseif (merctype==18) then
		skill=ML_PIERCE
		level=10
		sp=7
	elseif (merctype==20) then
		skill=ML_SPIRALPIERCE
		level=5
		sp=30
	elseif (merctype==21) then
		skill=MS_BASH
		level=1
		sp=8
	elseif (merctype==25) then
		skill=MS_BASH
		level=5
		sp=8
	elseif (merctype==27) then
		skill=MS_BASH
		level=10
		sp=15
	elseif (merctype==30) then
		skill=MS_BASH
		level=10
		sp=15
	end
	return skill,level,sp
end

function GetPushbackSkill(merctype)
	local skill = 0
	local level = 0
	local sp = 0
	local target = 1
	if (merctype==3) then
		skill=MA_CHARGEARROW
		level=1
		sp=15
	elseif (merctype==6) then
		skill=MA_SKIDTRAP
		level=3
		sp=15
		target = 2
	elseif (merctype==9) then
		skill=MA_CHARGEARROW
		level=1
		sp=15	
	elseif (merctype==10) then
		skill=MA_CHARGEARROW
		level=1
		sp=15
	end
	return skill,level,sp,target
end

function GetProvokeSkill(merctype)
	local skill = 0
	local level = 0
	local sp = 0
	local duration = 30000
	if (merctype==5) then
		skill=MER_PROVOKE
		level=1
		sp =4
	elseif (merctype==8) then
		skill=MER_PROVOKE
		level=3
		sp=6
	elseif (merctype==22) then
		skill=MER_PROVOKE
		level=5
		sp=8
	elseif (merctype==18) then
		skill=MER_PROVOKE
		level=5
		sp=8
	end
	return skill,level,sp,duration 
end

function GetSacrificeSkill(merctype)
	local skill = 0
	local level = 0
	local sp = 25
	local duration = 0
	if (merctype==13) then
		skill=ML_DEVOTION
		level=1
		duration=30000
	elseif (merctype==17) then
		skill=ML_DEVOTION
		level=1
		duration=30000
	elseif (merctype==20) then
		skill=ML_DEVOTION
		level=3
		duration=60000
	end
	return skill,level,sp,duration
end

function GetMobSkill(merctype)
	local skill = 0
	local level = 0
	local sp = 0
	local target = 1
	if (merctype==2) then
		skill=MA_SHOWER
		level=2
		sp=15
		target = 2
	elseif (merctype==7) then
		skill=MA_SHOWER
		level=10
		sp=15
		target = 2
	elseif (merctype==10) then
		skill=MA_SHARPSHOOTING
		level=5
		sp=30
	elseif (merctype==12) then
		skill=ML_BRANDISH
		level=2
		sp=12
	elseif (merctype==16) then
		skill=ML_BRANDISH
		level=5
		sp=12
	elseif (merctype==19) then
		skill=ML_BRANDISH
		level=10
		sp=12
	elseif (merctype==22) then
		skill=MS_MAGNUM
		level=3
		sp=30
		target=0
	elseif (merctype==24) then
		skill=MS_MAGNUM
		level=5
		sp=30
		target=0
	elseif (merctype==28) then
		skill=MS_BOWLINGBASH
		level=5
		sp=17
	elseif (merctype==29) then
		skill=MS_BOWLINGBASH
		level=8
		sp=20
	elseif (merctype==30) then
		skill=MS_BOWLINGBASH
		level=10
		sp=22
	end
	return skill,level,sp,target
end


function	GetQuickenSkill(merctype)
	local level = 0
	local skill = 0
	local sp = 0
	if (merctype==3) then
		level=1
	elseif (merctype==8) then
		level=2
	elseif (merctype==10) then
		level=5
	elseif (merctype==23) then
		level=1
	elseif (merctype==26) then
		level=5
	elseif (merctype==28) then
		level=10
	elseif (merctype==30) then
		level=10
	elseif (merctype==16) then
		level=2
	elseif (merctype==20) then
		level=5
	end
	local duration = level * 30000
	if (level ~= 0) then
		skill=MER_QUICKEN -- Can't just put this in all the time, need to return skill id 0 if the merc doesnt have quicken.
		sp=10+4*level
	end
	return skill,level,sp,duration
end

function	GetGuardSkill(merctype)
	local level = 0
	local skill = 0
	local duration = 300000
	local sp = 0
	if (merctype==15) then
		level=3
	elseif (merctype==19) then
		level=7
	elseif (merctype==20) then
		level=10
	end
	if (level ~= 0) then
		sp=10+2*level
		skill=ML_AUTOGUARD
	elseif (merctype==28) then
		level=4
		skill=MS_PARRYING
		sp=50
		duration=30000
	end
	--In practice, one should consider turning parrying off for the level 8 sword mercenary.
	return skill,level,sp,duration
end

